#import "LpushPlugin.h"
#import <LPushSDK/LPushSDK.h>
#import <LPushSDK/LPushNotificationContent.h>
#import <MOBFoundation/MOBFoundation.h>
#import <LPushSDK/LPushNotificationMessage.h>

@interface LpushPlugin()<FlutterStreamHandler>

// 事件回调
@property (nonatomic, copy) void (^callBack) (id _Nullable event);

@property (nonatomic, strong) NSMutableArray *tmps;

@end


@implementation LpushPlugin

static NSString *const receiverStr = @"lpush_receiver";


+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    //处理程序。确保使用的通道名称与 Flutter 客户端使用的一致。
    FlutterMethodChannel* channel = [FlutterMethodChannel
        methodChannelWithName:@"mob.com/lpush"
            binaryMessenger:[registrar messenger]];
    LpushPlugin* instance = [[LpushPlugin alloc] init];
    [registrar addMethodCallDelegate:instance channel:channel];
    
    FlutterEventChannel * e_channel = [FlutterEventChannel eventChannelWithName:receiverStr binaryMessenger:[registrar messenger]];
    [e_channel setStreamHandler:instance];
    
    [instance addObserver];
    

}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
  if ([@"getPlatformVersion" isEqualToString:call.method]) {
    result([@"iOS " stringByAppendingString:[[UIDevice currentDevice] systemVersion]]);
  }
    
    
  else if ([@"addLocalNotification" isEqualToString:call.method])
  {
      NSDictionary *arguments = (NSDictionary *)call.arguments;
      if (arguments && arguments[@"localNotification"])
      {
          NSString *localStr = arguments[@"localNotification"];
          
          NSDictionary *eventParams = [MOBFJson objectFromJSONString:localStr];
          
          NSString *identifier = nil;
          
          LPushNotificationContent *noti=[[LPushNotificationContent alloc] init];
          if (eventParams[@"title"] && ![eventParams[@"title"] isKindOfClass:[NSNull class]])
          {
              noti.title = eventParams[@"title"];
          }
          
          if (eventParams[@"content"] && ![eventParams[@"content"] isKindOfClass:[NSNull class]]) {
              noti.body = eventParams[@"content"];
          }
          if (eventParams[@"sound"] && ![eventParams[@"sound"] isKindOfClass:[NSNull class]])
          {
              noti.sound = eventParams[@"sound"];
              
          }
          if (eventParams[@"badge"] && ![eventParams[@"badge"] isKindOfClass:[NSNull class]])
          {
              noti.badge = eventParams[@"badge"];
          }
          if (eventParams[@"subTitle"] && ![eventParams[@"subTitle"] isKindOfClass:[NSNull class]])
          {
              noti.subTitle = eventParams[@"subTitle"];
          }
          
          NSMutableDictionary *userInfo = [NSMutableDictionary dictionary];
          if (eventParams[@"extrasMap"] && ![eventParams[@"extrasMap"] isKindOfClass:[NSNull class]])
          {
              [userInfo addEntriesFromDictionary:eventParams[@"extrasMap"]];
              
          }
          
          noti.userInfo = [userInfo copy];
          
          if (eventParams[@"identifier"] && ![eventParams[@"identifier"] isKindOfClass:[NSNull class]])
          {
              identifier = eventParams[@"messageId"];
          }
          
          LPushNotificationTrigger *trigger = [LPushNotificationTrigger new];

          if (eventParams[@"timestamp"] && ![eventParams[@"timestamp"] isKindOfClass:[NSNull class]])
          {
              double timeStamp = [eventParams[@"timestamp"] doubleValue];
              if (timeStamp > 0)
              {
                  NSDate *currentDate = [NSDate dateWithTimeIntervalSinceNow:0];
                  NSTimeInterval nowtime = [currentDate timeIntervalSince1970] * 1000;
                  NSTimeInterval tasktimeInterval = nowtime + (NSTimeInterval)timeStamp;
                  
                  if (@available(iOS 10.0, *))
                  {
                      trigger.timeInterval = timeStamp;
                  }
                  else
                  {
                      trigger.fireDate = [NSDate dateWithTimeIntervalSince1970:tasktimeInterval];
                  }
              }
          }
          
          LPushNotificationRequest *request = [LPushNotificationRequest requestWithIdentifier:identifier content:noti trigger:trigger];
          
          [LPushCenter addLocalNotification:request handler:^(id  _Nonnull result, NSError * _Nonnull error) {
              
              
              
          }];
      }
  }
    
  else if ([@"setBadge" isEqualToString:call.method])
  {
      NSDictionary *arguments = (NSDictionary *)call.arguments;
      NSInteger badge = [arguments[@"badge"] integerValue];
      UIApplication.sharedApplication.applicationIconBadgeNumber = badge;
      [LPushCenter setBadge:badge];
      
  }
    
  else if ([@"clearBadge" isEqualToString:call.method])
  {
      [LPushCenter clearBadge];
  }
  
  else if ([@"setCustomNotification" isEqualToString:call.method])
  {
      LPushNotificationConfiguration *config = [[LPushNotificationConfiguration alloc] init];
      config.types = LPushAuthorizationOptionSound | LPushAuthorizationOptionBadge | LPushAuthorizationOptionAlert;
      [[MOBFDataService sharedInstance] setCacheData:config forKey:@"LPushNotificationConfiguration" domain:@"LPUSH_FLUTTER_PLUGIN"];
      [LPushCenter registerForRemoteNotification:config];
  }
  else {
    result(FlutterMethodNotImplemented);
  }
}
#pragma mark - FlutterStreamHandler Protocol
- (FlutterError *)onListenWithArguments:(id)arguments eventSink:(FlutterEventSink)events
{
    self.callBack = events;
    return nil;
}

- (FlutterError * _Nullable)onCancelWithArguments:(id _Nullable)arguments
{
    return nil;
}




#pragma mark - 监听消息通知

-(void)addObserver
{
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didReceiveMessage:) name:LPushDidReceiveMessageNotification object:nil];
    LPushNotificationConfiguration *config = [[MOBFDataService sharedInstance] cacheDataForKey:@"LPushNotificationConfiguration" domain:@"LPUSH_FLUTTER_PLUGIN"];
    if (config && [config isKindOfClass:LPushNotificationConfiguration.class]) {
        [LPushCenter registerForRemoteNotification:config];
    }
    
}
- (void)didReceiveMessage:(NSNotification *)notification
{
    if([notification.object isKindOfClass:[LPushNotificationMessage class]])
    {
        LPushNotificationMessage *message = (LPushNotificationMessage *)notification.object;
        NSMutableDictionary *resultDict = [NSMutableDictionary dictionary];
        NSMutableDictionary *result = [NSMutableDictionary dictionary];
        switch (message.notificationMessageType) {
            case LPushNotificationMessageTypeAPNs:
            {//APNS回调
                if (message.content.userInfo) {
                    [result setObject:message.content.userInfo forKey:@"extrasMap"];
                }
                if (message.content.body) {
                    [result setObject:message.content.body forKey:@"content"];
                }
                [result addEntriesFromDictionary:message.content.convertDictionary];
                
                [resultDict setObject:@(0) forKey:@"action"];
                
                
            }
                break;
            case LPushNotificationMessageTypeLocal:
            {//本地通知回调
                if (message.content.userInfo) {
                    [result setObject:message.content.userInfo forKey:@"extrasMap"];
                    
                }
                
                if (message.content.body) {
                    [result setObject:message.content.body forKey:@"content"];
                }
                
                [result addEntriesFromDictionary:message.content.convertDictionary];
                
                [resultDict setObject:@(0) forKey:@"action"];
                
            }
                break;
            case LPushNotificationMessageTypeAPNsClicked:
            {
                if (message.content.userInfo) {
                    [result setObject:message.content.userInfo forKey:@"extrasMap"];
                    
                }
                
                if (message.content.body) {
                    [result setObject:message.content.body forKey:@"content"];
                }
                
                [result addEntriesFromDictionary:message.content.convertDictionary];
                
                [resultDict setObject:@(1) forKey:@"action"];
            }
                break;
            case LPushNotificationMessageTypeLocalClicked:
            {
                if (message.content.userInfo) {
                    [result setObject:message.content.userInfo forKey:@"extrasMap"];
                    
                }
                
                if (message.content.body) {
                    [result setObject:message.content.body forKey:@"content"];
                }
                
                [result addEntriesFromDictionary:message.content.convertDictionary];
                
                [resultDict setObject:@(1) forKey:@"action"];
            }
                break;
            default:
                break;
        }
        
        if (result.count > 0)
        {
            [resultDict setObject:result forKey:@"result"];
        }
        
        //回调结果
        NSString *resultDictStr = [MOBFJson jsonStringFromObject:resultDict];
        if (self.callBack)
        {
            self.callBack(resultDictStr);
        }
        else
        {
            if (_tmps)
            {
                [_tmps addObject:resultDictStr];
            }
            else
            {
                _tmps = @[resultDictStr].mutableCopy;
            }
        }
    }
    
}

- (void)setCallBack:(void (^)(id _Nullable))callBack
{
    _callBack = callBack;
    
    if (_callBack && _tmps.count) {
        for (NSString *obj in _tmps) {
            _callBack(obj);
        }
    }
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}
@end
