
import 'dart:async';
import 'dart:convert';

import 'package:flutter/services.dart';
import 'lpush_local_notification.dart';

typedef void EventHandler(Object event);

class Lpush {
  static const MethodChannel _channel =
      const MethodChannel('mob.com/lpush');
  static EventChannel _channelReciever = const EventChannel('lpush_receiver');


  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  /*
   * 添加推送回调监听（接收通知消息回调、接收点击通知消息回调、接收别名或标签操作回调）
   */
  static addPushReceiver(EventHandler onEvent,EventHandler onError){
      _channelReciever.receiveBroadcastStream().listen(onEvent,onError: onError);

  }

  /*
   * 移除推送回调监听
   */
  static Future<void> removePushReceiver() async{
    await _channel.invokeMethod('removePushReceiver');
  }

  /*
   * 发送本地通知
   */
  static Future<void> addLocalNotification(LPushLocalNotification localNotification) async{
    await _channel.invokeMethod('addLocalNotification', {"localNotification":json.encode(localNotification.toJson())});
  }

  /*
   * 设置远程推送，向用户授权
   */
  static Future<void> setCustomNotification() async{
    await _channel.invokeMethod('setCustomNotification');
  }

  /*
   * 设置角标 
   * @param badge  角标数量
   */
  static Future<void> setBadge(int badge) async{
    await _channel.invokeMethod('setBadge', {"badge": badge});
  }

  /*
   * 清空角标，不清除通知栏消息记录 (仅 iOS)
   */
  static Future<void> clearBadge() async{
    await _channel.invokeMethod('clearBadge');
  }

}

  


